/* 
 * Copyright  2004
 * Bankenes BetalingsSentral AS (BBS)
 * Organisation Number N- 975 946 231
 * Haavard Martinsens vei 54
 * N- 0045 OSLO
 * Norway
 */

#ifndef STRUCTURES_H
#define STRUCTURES_H

#ifdef WIN32
// Produce error instead of warnings for use of (possible) unitialized usage and undeclared functions.
// #pragma warning( disable:4706 ; once : 4267 4047 4996 ; error : 4002 4020 4700 4701 4013 4716 )  
#pragma warning( disable:4706 ; once : 4047 4996 ; error : 4002 4020 4700 4701 4013 4716 )  
#endif

/*	BANKID SDO STRUCTURE */

#define SDO_VERSION		"1.0"
#define SDO_POLICY		"2.16.578.1.16.4.1"
#define SDODATA_TEXT	"text/plain"
#define SDODATA_PDF		"application/pdf"
#define SDODATA_XML		"text/BIDXML"

/* BID_KeyValueElement - an element in the list of key/value-pairs */
typedef struct stBID_AttributeElement {
	char*					szKey;
	char*					szValue;
	struct stBID_AttributeElement*	nextKeyValuePair;
} BID_AttributeElement;


/* BID_Metadata - the data describing this SDOElement */
typedef struct stBID_Metadata {
	char*							szKeywords;
	char*							szDescription;
	struct stBID_AttributeElement*	attributeList;
} BID_Metadata;


/* BID_SignatureList - the list of signature elements */
typedef struct stBID_SignatureElement {
	char*					szDateTime; 
	char*					szSignature;
	unsigned char*					szOCSPResponse; 
	int						iOCSPResponse; 
	char*					szTimestamp;
	struct stBID_SignatureElement*	nextSignatureElement;	
} BID_SignatureElement;


/* BID_SDODatapart - the signed part of a BID_SDOElement */
typedef struct stBID_SDODatapart {
	char*					szPolicy;
	unsigned char*					szHashedData; 
	int						iHashedData; 
	char*					szDatatype;
	struct stBID_SignatureElement*	sigList;
} BID_SDODatapart;


/* BID_SDOElement - each "sub"-SDO in a complete SDO */
typedef struct stBID_SDOElement {
	char*				szBID_SDOVersion;
	struct stBID_SDODatapart*	sdoDatapart;
	char*				szSDOSignature;
	struct stBID_Metadata*			metadata;
	char*				szLocationURL;
	unsigned char*				szStoredData; 
	int					iStoredData; 
	struct stBID_SDOElement*	nextSDOElement;
} BID_SDOElement;


/* BankID_SDO - top level */
typedef struct stBID_SDO {
	struct stBID_SDOElement*	sdoList;
} BID_SDO;

/*	BANKID SDO STRUCTURE */




/*	SEID SDO STRUCTURE */
#define SEIDSDO_VERSION				 "1.0"
#define SEIDSDO_PROFILE				 "SEID-SDO-Basic-V"
#define SEIDSDO_SHA1ALGIDENTIFIER	 "SHA-1"
#define SEIDSDO_SHA256ALGIDENTIFIER	 "SHA-256"
#define SEIDSDO_POLICY				 "2.16.578.1.16.4.1"
#define SEIDSDO_POLICY_URN			 "urn:oid:2.16.578.1.16.4.1"
#define SEIDSDODATA_TEXT			 "text/plain"
#define SEIDSDODATA_PDF				 "application/pdf"
#define SEIDSDODATA_XML				 "text/BIDXML"


/********************* SDODATAPART ****************************************************/

typedef struct st_OCSPValues {
	char*											va_EncapsulatedOCSPValue;
} BID_SEIDSDOOCSPValues;

typedef struct st_RevocationValues {
	struct st_OCSPValues*							OCSPValues;
} BID_SEIDSDORevocationValues;

typedef struct st_UserCertificateAndRevocationData {
	struct st_RevocationValues*						RevocationValues;
} BID_SEIDSDOUserCertificateAndRevocationData;

typedef struct st_HashedData {
	char*											va_DigestMethod;
	char*											va_DigestValue;
} BID_SEIDSDOHashedData;

typedef struct st_SignersDocumentFormat {
	char*											va_MimeType;
} BID_SEIDSDOSignersDocumentFormat;

typedef struct st_SigPolicyId {
	char*											va_Identifier;
} BID_SEIDSDOSigPolicyId;

typedef struct st_SignaturePolicyId {
	struct st_SigPolicyId*							SigPolicyId;
} BID_SEIDSDOSignaturePolicyId;

typedef struct st_SignaturePolicyIdentifier {
	struct st_SignaturePolicyId*					SignaturePolicyId;
} BID_SEIDSDOSignaturePolicyIdentifier;


typedef struct st_CMSSignatureElement {
	char*											va_SDOProfile;
	struct st_SignaturePolicyIdentifier*			SignaturePolicyIdentifier;
	struct st_SignersDocumentFormat*				SignersDocumentFormat;
	struct st_HashedData*							HashedData;
	char*											va_CMSSignature;
	struct st_UserCertificateAndRevocationData*		UserCertificateAndRevocationData;
	
} BID_SEIDSDOCMSSignatureElement;

typedef struct st_SignatureElement {
	struct st_CMSSignatureElement*					CMSSignatureElement;
	struct st_SignatureElement*						nextSignatureElement;
} BID_SEIDSDOSignatureElement;

typedef struct st_SDODataPart {
	struct st_SignatureElement*						SignatureElement;
} BID_SEIDSDODataPart;
/********************* SDODATAPART ****************************************************/


/********************* SDOSEAL ********************************************************/
typedef struct st_SDOSignature {
	struct st_CMSSignatureElement*					CMSSignatureElement;
} BID_SEIDSDOSignature;

typedef struct st_SDOSeal {
	struct st_SDOSignature*							SDOSignature;
} BID_SEIDSDOSeal;
/********************* SDOSEAL ********************************************************/


/********************* METADATA ********************************************************/
typedef struct st_ValuePair{
	char*											va_Name;
	char*											va_Value;
	char*											va_Description;
	struct st_ValuePair*							nextValuePair;
} BID_SEIDSDOValuePair;

typedef struct st_MetaData{
	struct st_ValuePair*							ValuePair;
} BID_SEIDSDOMetaData;
/********************* METADATA ********************************************************/


/********************* SIGNEDOBJECT ****************************************************/
typedef struct st_SignedObject {
	char*											va_SignersDocument;
} BID_SEIDSDOSignedObject;
/********************* SIGNEDOBJECT ****************************************************/


/********************* THE SDO TOP ****************************************************/
typedef struct st_SEIDSDOElement {
	char*											va_SEIDSDOVersion;
	struct st_SDODataPart*							SDODataPart;
	struct st_SDOSeal*								SDOSeal;
	struct st_MetaData*								MetaData;
	struct st_SignedObject*							SignedObject;				
	struct st_SEIDSDOElement*						nextSEIDSDOElement;
	int												va_OldFormat;
} BID_SEIDSDOElement;

typedef struct st_SEIDSDO {
	struct st_SEIDSDOElement*						SEIDSDOElement;
} BID_SEIDSDO;
/********************* THE SDO TOP ****************************************************/

/*	SEID SDO STRUCTURE */


/*	SIGNATURE STRUCTURE */
typedef struct st_Signature {
	char*					pkcs7;
	char*					ocsp;
	struct	st_Signature*	next;
} BID_Signature;
/*	SIGNATURE STRUCTURE */

typedef struct st_SessionInfo {
	int						sessioncontext;
	char*					commonname;
	int						environment;
	char*					grantedpolicies;
	char*					merchantsigncert;
	char*					merchantauthcert;
	struct	st_SessionInfo*	next;
} BID_SessionInfo;
/*	SESSIONINFO STRUCTURE */

/********************************** Serial signing, PAdES extentions ******************/
/* ByteRange structure, four ints describing two separate disjunct ranges of
 * bytes. Corresponds to the ByteRange data structure from the PDF
 * specification.
 */
typedef struct stBID_ByteRange {
	int low0;
	int len0;
	int low1;
	int len1;
} BID_ByteRange;

/* BID_ByteRangeWithData structure, an array of bytes together with the
 * describing ByteRange.
 *
 * The application has to keep track of these (via BID_SerialSigningData)
 * between PAdES signing steps, so in a clustered environment, you might need to
 * serialize them. The length of the `data` element is given by `low1 + len1 -
 * low0`. Alternatively, call `bid_byterange_len_incl_gap` which performs that
 * computation for you.
 */
typedef struct stBID_ByteRangeWithData {
	BID_ByteRange byte_range;
	unsigned char * data;
} BID_ByteRangeWithData;

typedef enum enBID_SerialSigningKind
{
	BID_PARALLEL = 0, // Used internally 
	BID_SERIAL,
	BID_SERIAL_END_USER_ONLY
} BID_SerialSigningKind;

/* Certificate data sent to VisualSealBuilder, may or may not be used there. The dateOfBirth is null for merchants certificates */
typedef struct stBID_VisualSealCertData {
	char * commonName;
	char * serialNumber;
	char * organization;
	char * country;
	char * dateOfBirth;
} BID_VisualSealCertData;
/* Structure telling where seals shall be placed in turnkey mode, telling page in selfassembling mode */
typedef struct stBID_VisualSealPosition {
	double x;
	double y;
	int page;
} BID_VisualSealPosition;

/* Holds document data, kind of serial signing, the position of seals and the
 * incremental updates. The application has to hold on to instances of these
 * between steps in the signing flow.
 */
typedef struct stBID_SerialSigningData {
	BID_ByteRangeWithData * original_data;
	BID_SerialSigningKind signing_kind;
	BID_ByteRangeWithData *merchant_delta;
	BID_ByteRangeWithData *end_user_delta;
	BID_ByteRangeWithData *validation_delta;
	BID_VisualSealPosition *merchant_seal_position;
	BID_VisualSealPosition *end_user_seal_position;
} BID_SerialSigningData;

#if defined(_WIN64)
#define CALLBACKTYPE __stdcall
#else 
#define CALLBACKTYPE
#endif

/** Callback function used for building incremental updates containing visual seal for either or both of merchant and end user.
 This incremental update is returned as a ByteRangeWithData structure.

 The pdfData is the plain pdf data as far as it has been processed, not B64 encoded. The pdfData may for example
 contain the merchant seal and signature if endUserSeal is asked for, but will contain the initial pdf bytes if
 the merchant seal is asked for.

 The certData holds several values from the certificate to create incremental update for. This is a simple
 representation of the merchant or the end user.
 The resulting ByteRangeWithData may or may not contain the pdfData as prefix, that is: the byterange returned
 must EITHER start at 0 and the corresponding data bytes must include the pdfData from index 0 upto datalen and
 the incremental update from then on OR start at datalen and include the incremental update from then on.
 The incremental update must have a gap (described by the byteRange) where BankID server or BankID client
 will inserts the computed signature

 BankID server takes ownership of the data returned, i.e. freeing memory in result will be handled by bankID server.

 Must return 0 if successful, otherwise a non-zero error code that will be
 returned from InitTransactionSelfAssembler or initSignBuildEndUserSeal.
*/

typedef int(CALLBACKTYPE * BID_VisualSealBuilder)(int sessioncontext, int docnum,
	BID_ByteRangeWithData const * pdfdata,
	BID_VisualSealCertData const * visual_seal_cert_data,
	BID_ByteRangeWithData ** visual_seal_result);

/** Callback function used for adding OCSPs and CERTs element in a DSS in the PDF
* This function is used if the merchant does the validationdata packing and insertion itself
* It is allowed to set the validation_result to an empty byterange, but not null
* The validation_result and it's data element will be freed by BIDCServer
*
* ocsp and certs parameters are all Base64 encoded nullterminated strings of the DER encoding for the OCSPResponse or
* X509certificates. Certificate parameters are array of pointers to char, the last pointer is NULL, the array is never NULL
* endUserX509Certs[0] is the endUserCert as B64, endUserX509Certs[1] is the issuer and so on, normally the endUserX509Certs[2] == NULL
* same numbering for the merchantX509Certs, but if the document is BID_SERIAL_END_USER_ONLY then merchantCerts[0] == NULL
*
* Must return 0 if successful, otherwise a non-zero error code that will be
* returned from VerifyTransactionRequestSelfAssembler.
*/
typedef int(CALLBACKTYPE * BID_ValidationBuilder)(int sessioncontext, int docnum,
	BID_ByteRangeWithData const * pdfdata,
	char const * endUserFullOcspResponse,
	char const ** endUserX509Certs,
	char const * serverFullOcspResponse,
	char const ** merchantX509Certs,
	BID_ByteRangeWithData ** validation_result);

#endif
